@@ -8,33 +8,17 @@ package KinoSearch::Build::CBuilder;
 BEGIN { our @ISA = "ExtUtils::CBuilder"; }
 use Config;
 
-my %cc;
-
 sub new {
-    my ( $class, %args ) = @_;
+    my $class = shift;
     require ExtUtils::CBuilder;
-    if ( $ENV{KINO_VALGRIND} ) {
-        $args{config} ||= {};
-        $args{config}{optimize} ||= $Config{optimize};
-        $args{config}{optimize} =~ s/\-O\d+/-O1/g;
-    }
-    my $self = $class->SUPER::new(%args);
-    $cc{"$self"} = $args{'config'}->{'cc'};
-    return $self;
-}
-
-sub get_cc { $cc{"$_[0]"} }
-
-sub DESTROY {
-    my $self = shift;
-    delete $cc{"$self"};
+    return $class->SUPER::new(@_);
 }
 
 # This method isn't implemented by CBuilder for Windows, so we issue a basic
 # link command that works on at least one system and hope for the best.
 sub link_executable {
     my ( $self, %args ) = @_;
-    if ( $self->get_cc eq 'cl' ) {
+    if ( $Config{cc} eq 'cl' ) {
         my ( $objects, $exe_file ) = @args{qw( objects exe_file )};
         $self->do_system("link /out:$exe_file @$objects");
         return $exe_file;
@@ -63,11 +47,11 @@ sub xs_filepath { catfile( 'lib', "KinoSearch.xs" ) }
 sub autobind_pm_path { catfile( 'lib', 'KinoSearch', 'Autobinding.pm' ); }
 
 sub extra_ccflags {
-    my $self = shift;
+    my $self          = shift;
     my $extra_ccflags = defined $ENV{CFLAGS} ? "$ENV{CFLAGS} " : "";
     my $gcc_version 
         = $ENV{REAL_GCC_VERSION}
-        || $self->config('gccversion')
+        || $Config{gccversion}
         || undef;
     if ( defined $gcc_version ) {
         $gcc_version =~ /^(\d+(\.\d+))/
@@ -78,8 +62,7 @@ sub extra_ccflags {
     if ( defined $ENV{KINO_DEBUG} ) {
         if ( defined $gcc_version ) {
             $extra_ccflags .= "-DKINO_DEBUG ";
-            $extra_ccflags
-                .= "-DPERL_GCC_PEDANTIC -std=gnu99 -pedantic -Wall ";
+            $extra_ccflags .= "-DPERL_GCC_PEDANTIC -std=c99 -pedantic -Wall ";
             $extra_ccflags .= "-Wextra " if $gcc_version >= 3.4;    # correct
             $extra_ccflags .= "-Wno-variadic-macros "
                 if $gcc_version > 3.4;    # at least not on gcc 3.4
@@ -91,20 +74,20 @@ sub extra_ccflags {
     }
 
     # Compile as C++ under MSVC.
-    if ( $self->config('cc') eq 'cl' ) {
+    if ( $Config{cc} eq 'cl' ) {
         $extra_ccflags .= '/TP ';
     }
-
+    
     if ( defined $gcc_version ) {
-        # Tell GCC explicitly to run with maximum options.
-        if ( $extra_ccflags !~ m/-std=/ ) {
-            $extra_ccflags .= "-std=gnu99 ";
-        }
-        if ( $extra_ccflags !~ m/-D_GNU_SOURCE/ ) {
+        # Tell GCC explicitly to run with C99 compatability.
+        if ($extra_ccflags !~ m/-std=c99/) {
+            $extra_ccflags .= "-std=c99 ";
+        }  
+        if ($extra_ccflags !~ m/-D_GNU_SOURCE/) {
             $extra_ccflags .= "-D_GNU_SOURCE ";
-        }
+        } 
     }
-
+    
     return $extra_ccflags;
 }
 
@@ -146,8 +129,7 @@ sub ACTION_charmonizer {
 
     print "Building $CHARMONIZE_EXE_PATH...\n\n";
 
-    my $cbuilder = KinoSearch::Build::CBuilder->new(
-        config => { cc => $self->config('cc') }, );
+    my $cbuilder = KinoSearch::Build::CBuilder->new;
 
     my @o_files;
     for (@all_source) {
@@ -185,11 +167,10 @@ sub ACTION_charmony {
 
     # Clean up after Charmonizer if it doesn't succeed on its own.
     $self->add_to_cleanup("_charm*");
-    $self->add_to_cleanup($charmony_path);
 
     # Prepare arguments to charmonize.
-    my $cc        = $self->config('cc');
-    my $flags     = $self->config('ccflags') . ' ' . $self->extra_ccflags;
+    my $cc        = "$Config{cc}";
+    my $flags     = "$Config{ccflags} " . $self->extra_ccflags;
     my $verbosity = $ENV{DEBUG_CHARM} ? 2 : 1;
     $flags =~ s/"/\\"/g;
 
@@ -199,8 +180,8 @@ sub ACTION_charmony {
             and die "Failed to write charmony.h";
     }
     else {
-        system("./$CHARMONIZE_EXE_PATH \"$cc\" \"$flags\" $verbosity")
-            and die "Failed to write charmony.h: $!";
+        system("./$CHARMONIZE_EXE_PATH $cc \"$flags\" $verbosity")
+            and die "Failed to write charmony.h";
     }
 }
 
@@ -235,12 +216,12 @@ sub _compile_clownfish {
     }
 
     my $binding = Clownfish::Binding::Perl->new(
-        parcel     => 'KinoSearch',
-        hierarchy  => $hierarchy,
-        lib_dir    => 'lib',
-        boot_class => 'KinoSearch',
-        header     => $self->autogen_header,
-        footer     => $self->copyfoot,
+        parcel      => 'KinoSearch',
+        hierarchy   => $hierarchy,
+        lib_dir     => 'lib',
+        boot_class  => 'KinoSearch',
+        header      => $self->autogen_header,
+        footer      => $self->copyfoot,
     );
 
     return ( $hierarchy, $binding, \@pm_filepaths_with_xs );
@@ -276,18 +257,17 @@ sub ACTION_clownfish {
     }
     $self->add_to_cleanup($AUTOGEN_DIR);
 
-    my $pm_filepaths  = $self->rscan_dir( 'lib',            qr/\.pm$/ );
-    my $cfh_filepaths = $self->rscan_dir( $CORE_SOURCE_DIR, qr/\.cfh$/ );
+    my $pm_filepaths = $self->rscan_dir( 'lib',            qr/\.pm$/ );
+    my $bp_filepaths = $self->rscan_dir( $CORE_SOURCE_DIR, qr/\.bp$/ );
 
     # Don't bother parsing Clownfish files if everything's up to date.
     return
         if $self->up_to_date(
-        [ @$cfh_filepaths, @$pm_filepaths ],
-        [ $xs_filepath,    $AUTOGEN_DIR, ]
+        [ @$bp_filepaths, @$pm_filepaths ],
+        [ $xs_filepath,   $AUTOGEN_DIR, ]
         );
 
     # Write out all autogenerated files.
-    print "Parsing Clownfish files...\n";
     my ( $hierarchy, $perl_binding, $pm_filepaths_with_xs )
         = $self->_compile_clownfish;
     require Clownfish::Binding::Core;
@@ -306,7 +286,7 @@ sub ACTION_clownfish {
         $perl_binding->write_xs_typemap;
     }
 
-    # Rewrite XS if either any .cfh files or relevant .pm files were modified.
+    # Rewrite XS if either any .bp files or relevant .pm files were modified.
     $modified ||=
         $self->up_to_date( \@$pm_filepaths_with_xs, $xs_filepath )
         ? 0
@@ -324,14 +304,14 @@ sub ACTION_clownfish {
     # and didn't trigger a rewrite, so that we won't have to check them again
     # next pass.
     if (!$self->up_to_date(
-            [ @$cfh_filepaths, @$pm_filepaths_with_xs ], $xs_filepath
+            [ @$bp_filepaths, @$pm_filepaths_with_xs ], $xs_filepath
         )
         )
     {
         utime( time, time, $xs_filepath );    # touch
     }
     if (!$self->up_to_date(
-            [ @$cfh_filepaths, @$pm_filepaths_with_xs ], $AUTOGEN_DIR
+            [ @$bp_filepaths, @$pm_filepaths_with_xs ], $AUTOGEN_DIR
         )
         )
     {
@@ -360,7 +340,7 @@ sub ACTION_suppressions {
     my $LOCAL_SUPP = 'local.supp';
     return
         if $self->up_to_date( '../devel/bin/valgrind_triggers.pl',
-        $LOCAL_SUPP );
+                $LOCAL_SUPP );
 
     # Generate suppressions.
     print "Writing $LOCAL_SUPP...\n";
@@ -374,8 +354,8 @@ sub ACTION_suppressions {
     my $suppressions = `$command`;
     $suppressions =~ s/^==.*?\n//mg;
     my $rule_number = 1;
-    while ( $suppressions =~ /<insert.a.*?>/ ) {
-        $suppressions =~ s/^\s*<insert.a.*?>/{\n  <core_perl_$rule_number>/m;
+    while ( $suppressions =~ /<insert a.*?>/ ) {
+        $suppressions =~ s/^\s*<insert a.*?>/{\n  <core_perl_$rule_number>/m;
         $rule_number++;
     }
 
@@ -400,7 +380,7 @@ sub _valgrind_base_command {
 sub ACTION_test_valgrind {
     my $self = shift;
     die "Must be run under a perl that was compiled with -DDEBUGGING"
-        unless $self->config('ccflags') =~ /-D?DEBUGGING\b/;
+        unless $Config{ccflags} =~ /-D?DEBUGGING\b/;
     $self->dispatch('code');
     $self->dispatch('suppressions');
 
@@ -462,8 +442,7 @@ sub ACTION_compile_custom_xs {
 
     require ExtUtils::ParseXS;
 
-    my $cbuilder = KinoSearch::Build::CBuilder->new(
-        config => { cc => $self->config('cc') }, );
+    my $cbuilder = KinoSearch::Build::CBuilder->new;
     my $archdir = catdir( $self->blib, 'arch', 'auto', 'KinoSearch', );
     mkpath( $archdir, 0, 0777 ) unless -d $archdir;
     my @include_dirs = (
@@ -503,7 +482,7 @@ sub ACTION_compile_custom_xs {
     }
 
     # .c => .o
-    my $version = $self->dist_version;
+    my $version             = $self->dist_version;
     my $perl_binding_o_file = catfile( 'lib', "KinoSearch$Config{_o}" );
     unshift @objects, $perl_binding_o_file;
     $self->add_to_cleanup($perl_binding_o_file);
@@ -559,6 +538,27 @@ sub ACTION_code {
     $self->SUPER::ACTION_code;
 }
 
+# Copied from Module::Build::Base.pm, added exclude '#' and follow symlinks.
+sub rscan_dir {
+    my ( $self, $dir, $pattern ) = @_;
+    my @result;
+    local $_;    # find() can overwrite $_, so protect ourselves
+    my $subr
+        = !$pattern ? sub { push @result, $File::Find::name }
+        : !ref($pattern)
+        || ( ref $pattern eq 'Regexp' )
+        ? sub { push @result, $File::Find::name if /$pattern/ }
+        : ref($pattern) eq 'CODE'
+        ? sub { push @result, $File::Find::name if $pattern->() }
+        : die "Unknown pattern type";
+
+    File::Find::find( { wanted => $subr, no_chdir => 1, follow => 1 }, $dir );
+
+    # Skip emacs lock files.
+    my @filtered = grep !/#/, @result;
+    return \@filtered;
+}
+
 sub autogen_header {
     my $self = shift;
     return <<"END_AUTOGEN";
@@ -574,7 +574,7 @@ END_AUTOGEN
 }
 
 sub copyfoot {
-    return <<END_COPYFOOT;
+        return <<END_COPYFOOT;
 /* Copyright 2005-2010 Marvin Humphrey
  *
  * This program is free software; you can redistribute it and/or modify
@@ -623,8 +623,8 @@ sub _gen_pause_exclusion_list {
 
     my @excluded_files;
     for my $entry (@manifest_entries) {
-        # Allow README and Changes.
-        next if $entry =~ m#^(README|Changes)#;
+        # Allow README.
+        next if $entry =~ m#^README#;
 
         # Allow public modules.
         if ( $entry =~ m#^lib.+\.(pm|pod)$# ) {
@@ -638,13 +638,12 @@ sub _gen_pause_exclusion_list {
     }
 
     # Exclude redacted modules.
-    if ( eval { require "buildlib/KinoSearch/Redacted.pm" } ) {
-        my @redacted = map {
-            my @parts = split( /\W+/, $_ );
-            catfile( 'lib', @parts ) . '.pm'
-        } KinoSearch::Redacted->redacted, KinoSearch::Redacted->hidden;
-        push @excluded_files, @redacted;
-    }
+    require "buildlib/KinoSearch/Redacted.pm";
+    my @redacted = map {
+        my @parts = split( /\W+/, $_ );
+        catfile( 'lib', @parts ) . '.pm'
+    } KinoSearch::Redacted->redacted, KinoSearch::Redacted->hidden;
+    push @excluded_files, @redacted;
 
     my %uniquifier;
     @excluded_files = sort grep { !$uniquifier{$_}++ } @excluded_files;
@@ -655,9 +654,11 @@ sub _gen_pause_exclusion_list {
 
 __END__
 
-__COPYRIGHT__
+__POD__
+
+=head1 COPYRIGHT AND LICENSE
 
 Copyright 2005-2010 Marvin Humphrey
 
-This program is free software; you can redistribute it and/or modify
-under the same terms as Perl itself.
+=cut
+
