@@ -20,36 +20,52 @@ NumSortCache_init(NumericSortCache *self, Schema *schema, Folder *folder,
 {
     CharBuf   *field    = Seg_Field_Name(segment, field_num);
     CharBuf   *seg_name = Seg_Get_Name(segment);
-    CharBuf   *ord_file = CB_newf("%o/sort-%i32.ord", seg_name, field_num);
-    CharBuf   *dat_file = CB_newf("%o/sort-%i32.dat", seg_name, field_num);
     FieldType *type     = Schema_Fetch_Type(schema, field);
-    i32_t      doc_max  = Seg_Get_Count(segment);
-    i64_t ord_len, dat_len;
-    void *ords;
+    i32_t      doc_max  = (int32_t)Seg_Get_Count(segment);
+    i64_t      ord_len;
+    i64_t      dat_len;
+    void      *ords;
 
     /* Validate. */
-    if (!type || !FType_Sortable(type) || !OBJ_IS_A(type, NUMERICTYPE)) {
+    if (!type || !FType_Sortable(type) || !FType_Is_A(type, NUMERICTYPE)) {
+        DECREF(self);
         THROW(ERR, "'%o' isn't a sortable NumericType field", field);
     }
 
     /* Open instreams. */
-    self->ord_in  = Folder_Open_In(folder, ord_file);
-    self->dat_in  = Folder_Open_In(folder, dat_file);
-    if (!self->dat_in || !self->ord_in) {
-        CharBuf *mess = MAKE_MESS("Can't open either %o or %o for %o", 
-            ord_file, dat_file, field);
+    {
+        CharBuf *ord_file = CB_newf("%o/sort-%i32.ord", seg_name, field_num);
+        self->ord_in = Folder_Open_In(folder, ord_file);
         DECREF(ord_file);
+        if (!self->ord_in) {
+            Err *error = Err_new(CB_newf( 
+                "Error building sort cache for '%o': %o", field,
+                Err_get_error()));
+            DECREF(self);
+            Err_do_throw(error);
+        }
+    }
+    {
+        CharBuf *dat_file = CB_newf("%o/sort-%i32.dat", seg_name, field_num);
+        self->dat_in = Folder_Open_In(folder, dat_file);
         DECREF(dat_file);
-        Err_throw_mess(ERR, mess);
+        if (!self->dat_in) {
+            Err *error = Err_new(CB_newf( 
+                "Error building sort cache for '%o': %o", field,
+                Err_get_error()));
+            DECREF(self);
+            Err_do_throw(error);
+        }
     }
-    ord_len = InStream_Length(self->ord_in);
-    dat_len = InStream_Length(self->dat_in);
 
     /* Mmap ords and raw data. */
+    ord_len            = InStream_Length(self->ord_in);
+    dat_len            = InStream_Length(self->dat_in);
     ords               = InStream_Buf(self->ord_in, (size_t)ord_len);
     self->values       = InStream_Buf(self->dat_in, (size_t)dat_len);
     self->values_limit = self->values + dat_len;
 
+    /* Super-init. */
     SortCache_init((SortCache*)self, field, type, ords, num_unique, doc_max,
         null_ord);
 
@@ -59,6 +75,7 @@ NumSortCache_init(NumericSortCache *self, Schema *schema, Folder *folder,
         double docs_per_byte = BITS_PER_BYTE / self->ord_width;
         double max_ords      = ord_len * docs_per_byte;
         if (max_ords < self->doc_max + 1) {
+            DECREF(self);
             THROW(ERR, "Conflict between ord count max %f64 and doc_max %i32 for "
                 "field %o", max_ords, self->doc_max, field);
         }
@@ -70,14 +87,12 @@ NumSortCache_init(NumericSortCache *self, Schema *schema, Folder *folder,
         if (self->values_limit != limit) {
             i64_t wanted = limit - self->values;
             i64_t got    = self->values_limit - self->values;
+            DECREF(self);
             THROW(ERR, "Sort cache data file for %o should be %i64 bytes, "
                 "not %i64", field, wanted, got);
         }
     }
 
-    DECREF(ord_file);
-    DECREF(dat_file);
-
     ABSTRACT_CLASS_CHECK(self, NUMERICSORTCACHE);
     return self;
 }
@@ -115,7 +130,7 @@ F64SortCache_init(Float64SortCache *self, Schema *schema, Folder *folder,
 {
     CharBuf   *field = Seg_Field_Name(segment, field_num);
     FieldType *type  = Schema_Fetch_Type(schema, field);
-    if (!type || !FType_Sortable(type) || !OBJ_IS_A(type, FLOAT64TYPE)) {
+    if (!type || !FType_Sortable(type) || !FType_Is_A(type, FLOAT64TYPE)) {
         THROW(ERR, "'%o' isn't a sortable Float64Type field", field);
     }
     NumSortCache_init((NumericSortCache*)self, schema, folder, segment, 
@@ -134,7 +149,7 @@ F64SortCache_value(Float64SortCache *self, i32_t ord, Obj *blank)
     }
     else {
         char *val_ptr = self->values + ord * sizeof(double);
-        ASSERT_IS_A(blank, FLOAT64);
+        Float64 *num_blank = (Float64*)CERTIFY(blank, FLOAT64);
         if (val_ptr > self->values_limit) {
             i64_t over = val_ptr - self->values_limit;
             THROW(ERR, "Read %i64 beyond data limit for %o", over, self->field);
@@ -142,10 +157,10 @@ F64SortCache_value(Float64SortCache *self, i32_t ord, Obj *blank)
         #ifdef LITTLE_END
         {
             u64_t reversed_bytes = NumUtil_decode_bigend_u64(val_ptr);
-            Float64_Set_Value(blank, *((double*)&reversed_bytes));
+            Float64_Set_Value(num_blank, *((double*)&reversed_bytes));
         }
         #else 
-        Float64_Set_Value(blank, *(double*)val_ptr);
+        Float64_Set_Value(num_blank, *(double*)val_ptr);
         #endif
     }
     return blank;
@@ -170,7 +185,7 @@ F32SortCache_init(Float32SortCache *self, Schema *schema, Folder *folder,
 {
     CharBuf   *field = Seg_Field_Name(segment, field_num);
     FieldType *type  = Schema_Fetch_Type(schema, field);
-    if (!type || !FType_Sortable(type) || !OBJ_IS_A(type, FLOAT32TYPE)) {
+    if (!type || !FType_Sortable(type) || !FType_Is_A(type, FLOAT32TYPE)) {
         THROW(ERR, "'%o' isn't a sortable Float32Type field", field);
     }
     NumSortCache_init((NumericSortCache*)self, schema, folder, segment, 
@@ -189,7 +204,7 @@ F32SortCache_value(Float32SortCache *self, i32_t ord, Obj *blank)
     }
     else {
         char *val_ptr = self->values + ord * sizeof(float);
-        ASSERT_IS_A(blank, FLOAT32);
+        Float32 *num_blank = (Float32*)CERTIFY(blank, FLOAT32);
         if (val_ptr > self->values_limit) {
             i64_t over = val_ptr - self->values_limit;
             THROW(ERR, "Read %i64 beyond data limit for %o", over, self->field);
@@ -197,10 +212,10 @@ F32SortCache_value(Float32SortCache *self, i32_t ord, Obj *blank)
         #ifdef LITTLE_END
         {
             u32_t reversed_bytes = NumUtil_decode_bigend_u32(val_ptr);
-            Float32_Set_Value(blank, *((float*)&reversed_bytes));
+            Float32_Set_Value(num_blank, *((float*)&reversed_bytes));
         }
         #else 
-        Float32_Set_Value(blank, *(float*)val_ptr);
+        Float32_Set_Value(num_blank, *(float*)val_ptr);
         #endif
     }
     return blank;
@@ -225,7 +240,7 @@ I32SortCache_init(Int32SortCache *self, Schema *schema, Folder *folder,
 {
     CharBuf   *field = Seg_Field_Name(segment, field_num);
     FieldType *type  = Schema_Fetch_Type(schema, field);
-    if (!type || !FType_Sortable(type) || !OBJ_IS_A(type, INT32TYPE)) {
+    if (!type || !FType_Sortable(type) || !FType_Is_A(type, INT32TYPE)) {
         THROW(ERR, "'%o' isn't a sortable Int32Type field", field);
     }
     NumSortCache_init((NumericSortCache*)self, schema, folder, segment, 
@@ -244,12 +259,12 @@ I32SortCache_value(Int32SortCache *self, i32_t ord, Obj *blank)
     }
     else {
         char *val_ptr = self->values + ord * sizeof(i32_t);
-        ASSERT_IS_A(blank, INTEGER32);
+        Integer32 *int_blank = (Integer32*)CERTIFY(blank, INTEGER32);
         if (val_ptr > self->values_limit) {
             i64_t over = val_ptr - self->values_limit;
             THROW(ERR, "Read %i64 beyond data limit for %o", over, self->field);
         }
-        Int32_Set_Value(blank, NumUtil_decode_bigend_u32(val_ptr));
+        Int32_Set_Value(int_blank, NumUtil_decode_bigend_u32(val_ptr));
     }
     return blank;
 }
@@ -273,7 +288,7 @@ I64SortCache_init(Int64SortCache *self, Schema *schema, Folder *folder,
 {
     CharBuf   *field = Seg_Field_Name(segment, field_num);
     FieldType *type  = Schema_Fetch_Type(schema, field);
-    if (!type || !FType_Sortable(type) || !OBJ_IS_A(type, INT64TYPE)) {
+    if (!type || !FType_Sortable(type) || !FType_Is_A(type, INT64TYPE)) {
         THROW(ERR, "'%o' isn't a sortable Int64Type field", field);
     }
     NumSortCache_init((NumericSortCache*)self, schema, folder, segment, 
@@ -292,17 +307,17 @@ I64SortCache_value(Int64SortCache *self, i32_t ord, Obj *blank)
     }
     else {
         char *val_ptr = self->values + ord * sizeof(i64_t);
-        ASSERT_IS_A(blank, INTEGER64);
+        Integer64 *int_blank = (Integer64*)CERTIFY(blank, INTEGER64);
         if (val_ptr > self->values_limit) {
             i64_t over = val_ptr - self->values_limit;
             THROW(ERR, "Read %i64 beyond data limit for %o", over, self->field);
         }
-        Int64_Set_Value(blank, NumUtil_decode_bigend_u64(val_ptr));
+        Int64_Set_Value(int_blank, NumUtil_decode_bigend_u64(val_ptr));
     }
     return blank;
 }
 
-/* Copyright 2006-2009 Marvin Humphrey
+/* Copyright 2006-2010 Marvin Humphrey
  *
  * This program is free software; you can redistribute it and/or modify
  * under the same terms as Perl itself.
