@@ -1,3 +1,4 @@
+package KinoSearch::Index::TermInfo;
 use KinoSearch;
 
 1;
@@ -6,7 +7,7 @@ __END__
 
 __BINDING__
 
-Boilerplater::Binding::Perl::Class->register(
+Clownfish::Binding::Perl::Class->register(
     parcel       => "KinoSearch",
     class_name   => "KinoSearch::Index::TermInfo",
     bind_methods => [
@@ -27,7 +28,7 @@ Boilerplater::Binding::Perl::Class->register(
 
 __COPYRIGHT__
 
-Copyright 2006-2009 Marvin Humphrey
+Copyright 2006-2010 Marvin Humphrey
 
 This program is free software; you can redistribute it and/or modify
 under the same terms as Perl itself.
